;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: PWM8_2.asm
;;   Version: 2.1, Updated on 2003/08/26 at 16:06:45
;;  Generated by PSoC Designer ver 4.0 b865 : 27 August, 2003
;;
;;  DESCRIPTION: PWM8 User Module software implementation file
;;               for the 25/26xxx PSoC family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "PWM8_2.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
export  PWM8_2_EnableInt
export _PWM8_2_EnableInt
export  PWM8_2_DisableInt
export _PWM8_2_DisableInt
export  PWM8_2_Start
export _PWM8_2_Start
export  PWM8_2_Stop
export _PWM8_2_Stop
export  PWM8_2_WritePeriod
export _PWM8_2_WritePeriod
export  PWM8_2_WritePulseWidth
export _PWM8_2_WritePulseWidth
export  PWM8_2_bReadPulseWidth
export _PWM8_2_bReadPulseWidth
export  PWM8_2_bReadCounter
export _PWM8_2_bReadCounter

; The following functions are deprecated and subject to omission in future releases
;
export  bPWM8_2_ReadPulseWidth    ; deprecated
export _bPWM8_2_ReadPulseWidth    ; deprecated
export  bPWM8_2_ReadCounter       ; deprecated
export _bPWM8_2_ReadCounter       ; deprecated


AREA bss (RAM,REL)

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------

CONTROL_REG_START_BIT:         equ 0x01    ; Control register start bit
INPUT_REG_NULL:                equ 0x00    ; Clear the input register
INPUT_REG_CLOCK_MASK:          equ 0x0F    ; Input register clock mask


;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------


AREA UserModules (ROM, REL)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_EnableInt
;
;  DESCRIPTION:
;     Enables this PWM's interrupt by setting the interrupt enable mask bit
;     associated with this User Module. This function has no effect until and
;     unless the global interrupts are enabled (for example by using the
;     macro M8C_EnableGInt).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None.
;  RETURNS:      Nothing.
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_EnableInt:
_PWM8_2_EnableInt:
   M8C_EnableIntMask PWM8_2_INT_REG, PWM8_2_INT_MASK
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_DisableInt
;
;  DESCRIPTION:
;     Disables this PWM's interrupt by clearing the interrupt enable
;     mask bit associated with this User Module.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_DisableInt:
_PWM8_2_DisableInt:
   M8C_DisableIntMask PWM8_2_INT_REG, PWM8_2_INT_MASK
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_Start
;
;  DESCRIPTION:
;     Sets the start bit in the Control register of this user module.  The
;     PWM will begin counting on the next input clock as soon as the
;     enable input is asserted high.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_Start:
_PWM8_2_Start:
   or    reg[PWM8_2_CONTROL_REG], CONTROL_REG_START_BIT
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_Stop
;
;  DESCRIPTION:
;     Disables PWM operation by clearing the start bit in the Control
;     register.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_Stop:
_PWM8_2_Stop:
   and   reg[PWM8_2_CONTROL_REG], ~CONTROL_REG_START_BIT
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_WritePeriod
;
;  DESCRIPTION:
;     Write the 8-bit period value into the Period register (DR1).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: fastcall BYTE bPeriodValue (passed in A)
;  RETURNS:   Nothing
;  SIDE EFFECTS:
;     If the PWM user module is stopped, then this value will also be
;     latched into the Count register (DR0).
;     REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_WritePeriod:
_PWM8_2_WritePeriod:
   tst   reg[PWM8_2_CONTROL_REG], CONTROL_REG_START_BIT
   jnz   .PWMRunning

   ; The PWM is stopped.  We have to set the clock low for
   ; the write to the Period register to cause the data to be
   ; immediately transferred into the Count register, DR0.

   mov   X, A                                    ; X <- Period
   M8C_SetBank1
   mov   A, reg[PWM8_2_INPUT_REG]                ; A <- INPUT (Clock) setting
                                                 ; Force the clock signal low
   and   reg[PWM8_2_INPUT_REG], ~INPUT_REG_CLOCK_MASK
   M8C_SetBank0
   swap  A, X                                    ; A<-Period; X<-Clock
   mov   reg[PWM8_2_PERIOD_REG], A               ; Set the new Period
   mov   A, X                                    ; A<-Clock; X<-Clock
   M8C_SetBank1
   mov   reg[PWM8_2_INPUT_REG], A                ; Restore the clock
   M8C_SetBank0
   ret

   ; The PWM is running - write the period into the Period register.
   ; Upon Terminal Count this value will get loaded into the Count register.
.PWMRunning:
   mov   reg[PWM8_2_PERIOD_REG], A
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_WritePulseWidth
;
;  DESCRIPTION:
;     Writes compare value into the Compare register (DR2).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    fastcall BYTE bCompareValue (passed in A)
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_WritePulseWidth:
_PWM8_2_WritePulseWidth:
   mov   reg[PWM8_2_COMPARE_REG], A
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_bReadPulseWidth
;
;  DESCRIPTION:
;     Reads the Compare register.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      fastcall BYTE bCompareValue (value of DR2 in the A register)
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_bReadPulseWidth:
_PWM8_2_bReadPulseWidth:
 bPWM8_2_ReadPulseWidth:                         ; this name deprecated
_bPWM8_2_ReadPulseWidth:                         ; this name deprecated
   mov   A, reg[PWM8_2_COMPARE_REG]
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PWM8_2_bReadCounter
;
;  DESCRIPTION:
;     Returns the value in the Count register (DR0), preserving the value in
;     the compare register (DR2). Interrupts are prevented during the transfer
;     from the Count to the Compare registers by holding the clock low in
;     the PSoC block.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: None
;  RETURNS:   fastcall BYTE bCount (value of DR0 in the A register)
;  SIDE EFFECTS:
;     1) The user module is stopped momentarily and one or more counts may be missed.
;     2) REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 PWM8_2_bReadCounter:
_PWM8_2_bReadCounter:
 bPWM8_2_ReadCounter:                            ; this name deprecated
_bPWM8_2_ReadCounter:                            ; this name deprecated

   bOrigCompareValue:      EQU   0               ; Frame offset to temp Compare store
   bOrigClockSetting:      EQU   1               ; Frame offset to temp Input   store
   wCounter:               EQU   2               ; Frame offset to temp Count   store
   STACK_FRAME_SIZE:       EQU   3               ; max stack frame size is 3 bytes

   mov   X, SP                                   ; X <- stack frame pointer
   mov   A, reg[PWM8_2_COMPARE_REG]              ; Save the Compare register on the stack
   push  A                                       ;
                                                 ; Disable (stop) the PWM
   and   reg[PWM8_2_CONTROL_REG], ~CONTROL_REG_START_BIT
   M8C_SetBank1                                  ;
   mov   A, reg[PWM8_2_INPUT_REG]                ; save the clock input setting
   push  A                                       ;   on the stack (now 2 bytes) and ...
                                                 ;   hold the clock low:
   mov   reg[PWM8_2_INPUT_REG], INPUT_REG_NULL
   M8C_SetBank0
                                                 ; Extract the Count via DR2 register
   mov   A, reg[PWM8_2_COUNTER_REG]              ; DR2 <- DR0
   mov   A, reg[PWM8_2_COMPARE_REG]              ; Stash the Count on the stack
   push  A                                       ;  -stack frame is now 3 bytes
   mov   A, [X+bOrigCompareValue]                ; Restore the Compare register
   mov   reg[PWM8_2_COMPARE_REG], A
   M8C_SetBank1                                  ; Restore the PWM operation:
   mov   A, [X+bOrigClockSetting]                ;   First, the clock setting...
   mov   reg[PWM8_2_INPUT_REG], A                ;
   M8C_SetBank0                                  ;   then re-enable the PWM.
   or    reg[PWM8_2_CONTROL_REG], CONTROL_REG_START_BIT
   pop   A                                       ; Setup the return value
   ADD   SP, -(STACK_FRAME_SIZE-1)               ; Zap remainder of stack frame
   ret

.ENDSECTION

; End of File PWM8_2.asm
